/*
Copyright (C) 2009 -  Claudio Salvadori 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/**
 *\file IStreamDaq.h
 *
 *\date  Created on: 22-apr-2009
 *\author  Author: Claudio Salvadori
 *\brief Interface for different type of stream. This
 *	interface is the model for the lowest layer classes 
 *       that read data directly from differente devices. This 
*	class is used by the CReader class.
 */
#ifndef ISTREAMDAQ_H
#define ISTREAMDAQ_H

#include <vector>
#include <string>
#include <stdlib.h>

#include "Stream_enum.h"
#include "DAQ_struct.h"

/**\class IStreamDaq
*\brief Interface for different type of stream. This
 *	interface is the model for the lowest layer classes 
 *       that read data directly from differente devices. This 
*	class is used by the CReader class.*/

class IStreamDaq{
	public:
		virtual ~IStreamDaq() {};

		/**\brief Initialize the stream reader
		*\param def 	Pointer to a stream definition structure (input)
		*\param sty		Style of reading (by length, or by start and end character)
		*\return This method return one of the possible value from s_ret \see s_ret*/
		virtual s_ret getDefinition (struct SDef def, s_read_style sty) = 0;
		
		/**\brief Blocking method to wait the data to be passed to formatter.
		*\param buffer 	Pointer to a pointer of a vector of byte to be formatted (output)
		*\param length 	Pointer to the packet lenght variable (output)
		*\return This method return one of the possible value from s_ret \see s_ret*/
		virtual s_ret retStream(unsigned char **buffer, unsigned long *length) = 0;
		
};
#endif
