/*
Copyright (C) 2009 - Paolo Pagano
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/**
 *\file DAQSData.cxx
 *\author Paolo Pagano
 *\brief  DAQSData functions development
 */

/**
 *\file DAQSData.cpp
 *\author Paolo Pagano
 *\brief  Dstatic vector<IRenderer*>	rend;AQSData functions development (installed version) 
 */

/*TODO Check if included headers are needed inhere or in DAQEvent.cxx */
#include "TRandom.h"
#include "TDirectory.h"
#include "TProcessID.h"
#include "stdio.h"
#include <iostream>
#include "TH1D.h"
#include "TCanvas.h"
#include "globals.h"

#include "DAQEvent.h"

ClassImp(DAQSData)

DAQSData::DAQSData(const DAQSData &orig) : TObject(orig){

#ifdef DAQSOCKET1
	for(int i = 0; i < MESSAGE_BUFFER; i++)
		daqp.message_[i] = orig.daqp.message_[i];
		
	daqp.separator_ = orig.daqp.separator_;

	daqp.counter_ = orig.daqp.counter_;
#endif

#ifdef DAQSOCKET2
	for(int i = 0; i < MESSAGE_BUFFER; i++)
		daqp.message_[i] = orig.daqp.message_[i];
		
	daqp.separator1_ = orig.daqp.separator1_;

	daqp.random1_ = orig.daqp.random1_;
	
	daqp.separator2_ = orig.daqp.separator2_;

	daqp.random2_ = orig.daqp.random2_;
#endif

#ifdef DAQIMAGE
	daqp.width_ = orig.daqp.width_;
	daqp.height_ = orig.daqp.height_;
	daqp.duration_ = orig.daqp.duration_;
	daqp.lost_ = orig.daqp.lost_;
	daqp.ymean_ = orig.daqp.ymean_;
	memcpy ((UChar_t*)orig.daqp.image_, (UChar_t*)daqp.image_, IMAGE_BUFFER);
#endif

#ifdef DAQSNIFFER
	daqp.timestamp_=orig.daqp.timestamp_;
	memcpy ((UChar_t*)orig.daqp.payload_, (UChar_t*)daqp.payload_, PAYLOAD_BUFFER);
#endif

#ifdef DAQTIMERAVR
	daqp.cycles_[0] = orig.daqp.cycles_[0];
	daqp.cycles_[1] = orig.daqp.cycles_[1];
	daqp.cycles_[2] = orig.daqp.cycles_[2];
	daqp.cycles_[3] = orig.daqp.cycles_[3];

	daqp.tn_[0] = orig.daqp.tn_[0];
	daqp.tn_[1] = orig.daqp.tn_[1];
	daqp.tn_[2] = orig.daqp.tn_[2];
	daqp.tn_[3] = orig.daqp.tn_[3];

	daqp.seqnum_ = orig.daqp.seqnum_;
	daqp.pkt_length_ = orig.daqp.pkt_length_;
#endif

#ifdef DAQTIMERAVR_EE

	daqp.EtoE_[0] = orig.daqp.EtoE_[0];
	daqp.SeqNum_[0] = orig.daqp.SeqNum_[0];
	daqp.Pkt_Length_[0] = orig.daqp.Pkt_Length_[0];
#endif

#ifdef DAQTIMERUW_EE
	daqp.time_ = orig.daqp.time_;
	daqp.length_ = orig.daqp.length_;
#endif

#ifdef DAQTIMERUW
	daqp.time_ = orig.daqp.time_;
	daqp.length_ = orig.daqp.length_;
#endif

#ifdef DAQTRACKING
	memcpy(daqp.position_, orig.daqp.position_, 4);
#endif

#ifdef DAQMIRTES
	daqp.counter_ = orig.daqp.counter_;
	daqp.minimum_ = orig.daqp.minimum_;
	daqp.maximum_ = orig.daqp.maximum_;
	daqp.average_ = orig.daqp.average_;
#endif

#ifdef DAQSTEREO
	memcpy ((UChar_t*)orig.daqp.image_dx_, (UChar_t*)daqp.image_dx_, STEREO_BUFFER);
	memcpy ((UChar_t*)orig.daqp.image_sx_, (UChar_t*)daqp.image_sx_, STEREO_BUFFER);
	memcpy ((UChar_t*)orig.daqp.image_stereo_, (UChar_t*)daqp.image_stereo_, STEREO_BUFFER);
#endif
}
//______________________________________________________________________________
DAQSData::DAQSData() : TObject(){
#ifdef DAQSOCKET1
	for(int i = 0; i < MESSAGE_BUFFER; i++)
		daqp.message_[i] = 0;
		
	daqp.separator_ = 0;
	
	daqp.counter_ = 0;
#endif

#ifdef DAQSOCKET2
	for(int i = 0; i < MESSAGE_BUFFER; i++)
		daqp.message_[i] = 0;
		
	daqp.separator1_ = 0;

	daqp.random1_ = 0;
	
	daqp.separator2_ = 0;

	daqp.random2_ = 0;
#endif

#ifdef DAQIMAGE
	daqp.width_ = 0;	
	daqp.height_ = 0;
	daqp.duration_ = 0;
	daqp.lost_ = 0;
	daqp.ymean_ = 0;
	memset(daqp.image_, '\0', IMAGE_BUFFER);
#endif

#ifdef DAQSNIFFER
	daqp.timestamp_ = 0;
	memset(daqp.payload_, '\0', PAYLOAD_BUFFER);
#endif

#ifdef DAQTIMERAVR
	daqp.cycles_[0] = 0;
	daqp.cycles_[1] = 0;
	daqp.cycles_[2] = 0;
	daqp.cycles_[3] = 0;

	daqp.static vector<IRenderer*>	rend;tn_[0] = 0;
	daqp.tn_[1] = 0;
	daqp.tn_[2] = 0;
	daqp.tn_[3] = 0;

	daqp.seqnum_ = 0;
	daqp.pkt_length_ = 0;
#endif
#ifdef DAQTIMERAVR_EE
	daqp.EtoE_[0] = 0;
	daqp.SeqNum_[0] = 0;
	daqp.Pkt_Length_[0] = 0;

#endif

#ifdef DAQTIMERUW_EE
	daqp.time_ = 0;
	daqp.length_ = 0;
#endif

#ifdef DAQTIMERUW
	daqp.time_ = 0;
	daqp.length_ = 0;
#endif

#ifdef DAQTRACKING
	daqp.position_[0] = 0;
	daqp.position_[1] = 0;
#endif

#ifdef DAQMIRTES
	daqp.counter_ = 0;
	daqp.minimum_ = 0;
	daqp.maximum_ = 0;
	daqp.average_ = 0;
#endif

#ifdef DAQSTEREO
	memset(daqp.image_dx_, '\0', STEREO_BUFFER);
	memset(daqp.image_sx_, '\0', STEREO_BUFFER);
	memset(daqp.image_stereo_, '\0', STEREO_BUFFER);
#endif
}
//______________________________________________________________________________

DAQSData::DAQSData(std::vector<SData> sdata_) : TObject(){

#ifdef DAQSOCKET1
	for(unsigned int i = 0; i < sdata_.size(); i++){
		if(sdata_[i].d_Name == "Message"){
			cerr <<  sdata_[i].d_Name.c_str() << ": ";
			for(unsigned int j = 0; j < MESSAGE_BUFFER - 1; j++)
				daqp.message_[j] = *((Char_t*)(sdata_[i].d_Data) + j);
			daqp.message_[MESSAGE_BUFFER - 1] = (Char_t)'\0';
			cerr << (char*) daqp.message_ << endl;	
			IRenderer::getRenderer(i)->render((void*) daqp.message_, sdata_[i].d_Size);
		}else if(sdata_[i].d_Name == "Separator"){
			cerr <<  sdata_[i].d_Name.c_str() << ": ";
			daqp.separator_ = *(unsigned int*)(sdata_[i].d_Data);
			cerr <<  daqp.separator_ << endl;		
			IRenderer::getRenderer(i)->render((void*) &daqp.separator_, sdata_[i].d_Size);
		}else if(sdata_[i].d_Name == "Counter"){
			cerr <<  sdata_[i].d_Name.c_str() << ": ";
			daqp.counter_ = *(unsigned int*)(sdata_[i].d_Data);
			cerr <<  daqp.counter_ << endl;		
			IRenderer::getRenderer(i)->render((void*) &daqp.counter_, sdata_[i].d_Size);
		}
	}
#endif

#ifdef DAQSOCKET2
	for(unsigned int i = 0; i < sdata_.size(); i++){
		if(sdata_[i].d_Name == "Message"){
			cerr <<  sdata_[i].d_Name.c_str() << ": ";
			for(unsigned int j = 0; j < MESSAGE_BUFFER - 1; j++)
				daqp.message_[j] = *((Char_t*)(sdata_[i].d_Data) + j);
			daqp.message_[MESSAGE_BUFFER - 1] = (Char_t)'\0';
			cerr << (char*) daqp.message_ << endl;	
			IRenderer::getRenderer(i)->render((void*) daqp.message_, sdata_[i].d_Size);
		}else if(sdata_[i].d_Name == "Separator1"){
			cerr <<  sdata_[i].d_Name.c_str() << ": ";
			daqp.separator1_ = *(unsigned int*)(sdata_[i].d_Data);
			cerr <<  daqp.separator1_ << endl;		
			IRenderer::getRenderer(i)->render((void*) &daqp.separator1_, sdata_[i].d_Size);
		}else if(sdata_[i].d_Name == "Random1"){
			cerr <<  sdata_[i].d_Name.c_str() << ": ";
			daqp.random1_ = *(unsigned int*)(sdata_[i].d_Data);
			cerr <<  daqp.random1_ << endl;		
			IRenderer::getRenderer(i)->render((void*) &daqp.random1_, sdata_[i].d_Size);
		}else if(sdata_[i].d_Name == "Separator2"){
			cerr <<  sdata_[i].d_Name.c_str() << ": ";
			daqp.separator2_ = *(unsigned int*)(sdata_[i].d_Data);
			cerr <<  daqp.separator2_ << endl;		
			IRenderer::getRenderer(i)->render((void*) &daqp.separator2_, sdata_[i].d_Size);
		}else if(sdata_[i].d_Name == "Random2"){
			cerr <<  sdata_[i].d_Name.c_str() << ": ";
			daqp.random2_ = *(unsigned int*)(sdata_[i].d_Data);
			cerr <<  daqp.random2_ << endl;		
			IRenderer::getRenderer(i)->render((void*) &daqp.random2_, sdata_[i].d_Size);
		}
	}
#endif	

#ifdef DAQIMAGE
	for(unsigned int i = 0; i < sdata_.size(); i++){
		if(sdata_[i].d_Name == "Width"){
			cerr <<  sdata_[i].d_Name.c_str() << ": ";
			daqp.width_ = *((UShort_t*)(sdata_[i].d_Data));
			cerr <<  daqp.width_ << endl;
			IRenderer::getRenderer(i)->render((void*) &daqp.width_, sdata_[i].d_Size);
		}else if(sdata_[i].d_Name == "Height"){
			cerr <<  sdata_[i].d_Name.c_str() << ": ";
			daqp.height_ = *(UShort_t*)(sdata_[i].d_Data);
			cerr <<  daqp.height_ << endl;
			IRenderer::getRenderer(i)->render((void*) &daqp.height_, sdata_[i].d_Size);
		}else if(sdata_[i].d_Name == "Duration"){
			cerr <<  sdata_[i].d_Name.c_str() << ": ";
			daqp.duration_ = *(UInt_t*)(sdata_[i].d_Data);
			cerr <<  daqp.duration_ << endl;
			IRenderer::getRenderer(i)->render((void*) &daqp.duration_, sdata_[i].d_Size);
		}else if(sdata_[i].d_Name == "Lost"){
			cerr <<  sdata_[i].d_Name.c_str() << ": ";
			daqp.lost_ = *(UShort_t*)(sdata_[i].d_Data);
			cerr <<  daqp.lost_ << endl;
			IRenderer::getRenderer(i)->render((void*) &daqp.lost_, sdata_[i].d_Size);
		}else if(sdata_[i].d_Name == "YMean"){
			cerr <<  sdata_[i].d_Name.c_str() << ": ";
			daqp.ymean_ = *(UShort_t*)(sdata_[i].d_Data);
			cerr <<  daqp.ymean_ << endl;
			IRenderer::getRenderer(i)->render((void*) &daqp.ymean_, sdata_[i].d_Size);
		}else if(sdata_[i].d_Name == "Image"){
			cerr <<  sdata_[i].d_Name.c_str() << ": ";
			memcpy(daqp.image_, (UChar_t*)sdata_[i].d_Data, IMAGE_BUFFER);	
			IRenderer::getRenderer(i)->render((void*) daqp.image_, sdata_[i].d_Size);
			cerr << endl;
		}
	}
#endif

#ifdef DAQSNIFFER	
	for(unsigned int i = 0; i < sdata_.size(); i++){
		if(sdata_[i].d_Name == "Timestamp"){
			cerr <<  sdata_[i].d_Name.c_str() << ": ";
			daqp.timestamp_ = *(UInt_t*)(sdata_[i].d_Data);
			cerr << dec <<  daqp.timestamp_ << endl;
			IRenderer::getRenderer(i)->render((void*) &daqp.timestamp_, sdata_[i].d_Size);	
		}else if(sdata_[i].d_Name == "Payload"){
			cerr <<  sdata_[i].d_Name.c_str() << ": ";
			memcpy(daqp.payload_, (UChar_t*)sdata_[i].d_Data, PAYLOAD_BUFFER);	
			IRenderer::getRenderer(i)->render((void*)daqp.payload_, sdata_[i].d_Size);
			/*for (int k =0; k < PAYLOAD_BUFFER; k++){
				cerr << hex << (unsigned int)daqp.payload_[k] << " ";
				if (k % 10 == 0) cerr << endl;
			}*/
			cerr << endl;			
		}
	}
#endif

#ifdef DAQTIMERAVR	
	for(unsigned int i = 0; i < sdata_.size(); i++){
		if(sdata_[i].d_Name == "Cycles_1"){
			printf("%s: ", sdata_[i].d_Name.c_str());
			daqp.cycles_[0] = *(UShort_t*)(sdata_[i].d_Data);
			printf("%u, \n ", daqp.cycles_[0]);	
		}else if(sdata_[i].d_Name == "Cycles_2"){
			printf("%s: ", sdata_[i].d_Name.c_str());
			daqp.cycles_[1] = *(UShort_t*)(sdata_[i].d_Data);
			printf("%u, \n ", daqp.cycles_[1]);			
		}else if(sdata_[i].d_Name == "Cycles_3"){
			printf("%s: ", sdata_[i].d_Name.c_str());
			daqp.cycles_[2] = *(UShort_t*)(sdata_[i].d_Data);
			printf("%u, \n ", daqp.cycles_[2]);
		}else if(sdata_[i].d_Name == "Cycles_4"){
			printf("%s: ", sdata_[i].d_Name.c_str());
			daqp.cycles_[3] = *(UShort_t*)(sdata_[i].d_Data);
			printf("%u, \n ", daqp.cycles_[3]);
		}else if(sdata_[i].d_Name == "TN_1"){
			printf("%s: ", sdata_[i].d_Name.c_str());
			daqp.tn_[0] = *(UShort_t*)(sdata_[i].d_Data);
			printf("%u, \n ", daqp.tn_[0]);
		}else if(sdata_[i].d_Name == "TN_2"){
			printf("%s: ", sdata_[i].d_Name.c_str());
			daqp.tn_[1] = *(UShort_t*)(sdata_[i].d_Data);
			printf("%u, \n ", daqp.tn_[1]);
		}else if(sdata_[i].d_Name == "TN_3"){
			printf("%s: ", sdata_[i].d_Name.c_str());
			daqp.tn_[2] = *(UShort_t*)(sdata_[i].d_Data);
			printf("%u, \n ", daqp.tn_[2]);
		}else if(sdata_[i].d_Name == "TN_4"){
			printf("%s: ", sdata_[i].d_Name.c_str());
			daqp.tn_[3] = *(UShort_t*)(sdata_[i].d_Data);
			printf("%u, \n ", daqp.tn_[3]);
		}else if(sdata_[i].d_Name == "SeqNum"){
			printf("%s: ", sdata_[i].d_Name.c_str());
			daqp.seqnum_ = *(UShort_t*)(sdata_[i].d_Data);
			printf("%u, \n ", daqp.seqnum_);
		}else if(sdata_[i].d_Name == "Pkt_Length"){
			printf("%s: ", sdata_[i].d_Name.c_str());
			daqp.pkt_length_ = *(UShort_t*)(sdata_[i].d_Data);
			printf("%u, \n ", daqp.pkt_length_);
		}
	}
#endif

#ifdef DAQTIMERAVR_EE
	for(unsigned int i = 0; i < sdata_.size(); i++){
		if(sdata_[i].d_Name == "EtoE"){
			printf("%s: ", sdata_[i].d_Name.c_str());
			daqp.EtoE_[0] = *(UInt_t*)(sdata_[i].d_Data);
			printf("%u, \n ", daqp.EtoE_[0]);	
		}
		else if(sdata_[i].d_Name == "SeqNum"){
			printf("%s: ", sdata_[i].d_Name.c_str());
			daqp.SeqNum_[0] = *(UShort_t*)(sdata_[i].d_Data);
			printf("%u, \n ", daqp.SeqNum_[0]);
		}else if(sdata_[i].d_Name == "Pkt_Length"){
			printf("%s: ", sdata_[i].d_Name.c_str());
			daqp.Pkt_Length_[0] = *(UChar_t*)(sdata_[i].d_Data);
			printf("%u, \n ", daqp.Pkt_Length_[0]);
		}
	}
#endif

#ifdef DAQTIMERUW_EE
	for(unsigned int i = 0; i < sdata_.size(); i++){
		if(sdata_[i].d_Name == "Time"){
			printf("%s: ", sdata_[i].d_Name.c_str());
			daqp.time_ = *(UShort_t*)(sdata_[i].d_Data);
			printf("%u, \n ", daqp.time_);	
		}else if(sdata_[i].d_Name == "Length"){
			printf("%s: ", sdata_[i].d_Name.c_str());
			daqp.length_ = *(UShort_t*)(sdata_[i].d_Data);
			printf("%u, \n ", daqp.length_);			
		}
	}
#endif

#ifdef DAQTIMERUW
	for(unsigned int i = 0; i < sdata_.size(); i++){
		if(sdata_[i].d_Name == "Time"){
			printf("%s: ", sdata_[i].d_Name.c_str());
			daqp.time_ = *(UShort_t*)(sdata_[i].d_Data);
			printf("%u, \n ", daqp.time_);	
		}else if(sdata_[i].d_Name == "Length"){
			printf("%s: ", sdata_[i].d_Name.c_str());
			daqp.length_ = *(UShort_t*)(sdata_[i].d_Data);
			printf("%u, \n ", daqp.length_);			
		}
	}
#endif

#ifdef DAQTRACKING
	for(unsigned int i = 0; i < sdata_.size(); i++){
		if(sdata_[i].d_Name == "Position"){
			cerr << sdata_[i].d_Name.c_str() << ": ";
			memcpy(daqp.position_, ((UShort_t*)sdata_[i].d_Data), sdata_[i].d_Size * INT16_S);
			cerr << "x = " << daqp.position_[0] << " y = " << daqp.position_[1] << endl;
			IRenderer::getRenderer(i)->render((void*) daqp.position_, sdata_[i].d_Size);
		}
	}
#endif

#ifdef DAQMIRTES
	static int mycounter = 0;
	for(unsigned int i = 0; i < sdata_.size(); i++){
		if(sdata_[i].d_Name == "Counter"){
			printf("%s: ", sdata_[i].d_Name.c_str());
			daqp.counter_ = *(UShort_t*)(sdata_[i].d_Data);
			printf("%u, \n ", daqp.counter_);	
		}else if(sdata_[i].d_Name == "Minimum"){
			printf("%s: ", sdata_[i].d_Name.c_str());
			daqp.minimum_ = *(UShort_t*)(sdata_[i].d_Data);
			printf("%u, \n ", daqp.minimum_);
		}else if(sdata_[i].d_Name == "Maximum"){
			printf("%s: ", sdata_[i].d_Name.c_str());
			daqp.maximum_ = *(UShort_t*)(sdata_[i].d_Data);
			printf("%u, \n ", daqp.maximum_);
		}else if(sdata_[i].d_Name == "Average"){
			printf("%s: ", sdata_[i].d_Name.c_str());
			daqp.average_ = *(Float_t*)(sdata_[i].d_Data);
			printf("%f, \n ", daqp.average_);			
		}
	
		switch(sdata_[i].d_Sem){
		case DS_IMAGE:
			sdata_[i].d_Renderer->render(sdata_[i].d_Data);
			break; 
		case DS_HISTOGRAM:
			sdata_[i].d_Renderer->render(sdata_[i].d_Data);
			break;	
		case DS_GRAPH:
			sdata_[i].d_Renderer->render(sdata_[i].d_Data);
			break;
		default:
			break;
		}
	}
#endif

#ifdef DAQSTEREO
	for(unsigned int i = 0; i < sdata_.size(); i++){
		 if(sdata_[i].d_Name == "Image_sx"){
			//fprintf(stderr, "%s: ", sdata_[i].d_Name.c_str());
			memcpy(daqp.image_sx_, (UChar_t*)sdata_[i].d_Data, STEREO_BUFFER);	
			initRendererSX(STEREO_WIDTH, STEREO_HEIGHT);
			renderImageSX((unsigned char*) sdata_[i].d_Data, STEREO_WIDTH, STEREO_HEIGHT, 1);	
		}else if(sdata_[i].d_Name == "Image_dx"){
			//fprintf(stderr, "%s: ", sdata_[i].d_Name.c_str());
			memcpy(daqp.image_dx_, (UChar_t*)sdata_[i].d_Data, STEREO_BUFFER);	
			initRendererDX(STEREO_WIDTH, STEREO_HEIGHT);
			renderImageDX((unsigned char*) sdata_[i].d_Data, STEREO_WIDTH, STEREO_HEIGHT, 1);
		}else if(sdata_[i].d_Name == "Image_stereo"){	
			//fprintf(stderr, "%s: ", sdata_[i].d_Name.c_str());
			memcpy(daqp.image_stereo_, (UChar_t*)sdata_[i].d_Data, STEREO_BUFFER);	
			initRendererDisp(STEREO_WIDTH, STEREO_HEIGHT);
			renderImageDisp((unsigned char*) sdata_[i].d_Data, STEREO_WIDTH, STEREO_HEIGHT, 1);	
		}
	}

	unsigned char h_sx[GRAY_LEVEL], h_dx[GRAY_LEVEL];

	memset(h_sx, 0, GRAY_LEVEL);
	memset(h_dx, 0, GRAY_LEVEL);

	myhisto_[0]->Reset("ICEM");
	myhisto_[1]->Reset("ICEM");

	TAxis *ax[2]; 
	ax[0]= myhisto_[0]->GetXaxis();
	ax[1]= myhisto_[1]->GetXaxis();
	
	create_histogram(daqp.image_sx_, STEREO_BUFFER, h_sx);
	create_histogram(daqp.image_dx_, STEREO_BUFFER, h_dx);

	for(unsigned int i = 0; i < STEREO_BUFFER; i++){
		
		myhisto_[0]->SetBinContent(myhisto_[0]->FindBin(i), 
					h_sx[i]);
		myhisto_[1]->SetBinContent(myhisto_[1]->FindBin(i), 
					h_dx[i]);
	}

	c[0]->Modified();
	c[0]->Update();

	c[1]->Modified();
	c[1]->Update();

#endif
}
//______________________________________________________________________________
void DAQSData::printData() const{

#ifdef DAQSOCKET1
	cerr << "DAQSOCKET1 printData: " << endl;
	cerr << "Message: ";
	for(unsigned int i = 0; i < MESSAGE_BUFFER - 1; i++)
		cerr << *(((char*)daqp.message_) + i) << ", ";
	cerr << endl;
	
	cerr << "Separator: " << daqp.separator_ << endl;
	
	cerr << "Counter: " << daqp.counter_ << endl;
#endif

#ifdef DAQSOCKET2
	cerr << "DAQSOCKET2 printData: " << endl;
	cerr << "Message: ";
	for(unsigned int i = 0; i < MESSAGE_BUFFER - 1; i++)
		cerr << *(((char*)daqp.message_) + i) << ", ";
	cerr << endl;
	
	cerr << "Separator 1: " << daqp.separator1_ << endl;
	
	cerr << "Random 1: " << daqp.random1_ << endl;
	
	cerr << "Separator 2: " << daqp.separator2_ << endl;
	
	cerr << "Random 2: " << daqp.random2_ << endl;
#endif

#ifdef DAQIMAGE
	cerr << "DAQIMAGE printData: " << endl;
	cerr << "Width: " << daqp.width_ << endl;
	cerr << "Height: " << daqp.height_ << endl;
	cerr << "Duration: " << daqp.duration_ << endl;
	cerr << "Lost: " << daqp.lost_ << endl;
	cerr << "YMean: " << daqp.ymean_ << endl;
	cerr << "Image: " << endl;
	for (int k =0; k < IMAGE_BUFFER; k++){
		cerr << hex << (unsigned int)daqp.image_[k] << " ";
		if (k % 10 == 0) cerr << endl;
	}
	cerr << endl;
#endif

#ifdef DAQSNIFFER
	cerr << "DAQSNIFFER printData: " << endl;
	cerr << "Time-stamp: : " << daqp.timestamp_ << endl;
	cerr << "Payload: " << endl;
	for (int k =0; k < PAYLOAD_BUFFER; k++){
		cerr << hex << (unsigned int)daqp.payload_[k] << " ";
		if (k % 10 == 0) cerr << endl;
	}
	cerr << endl;
#endif

#ifdef DAQTIMERAVR
	printf("Cycles[0]: %u\n",  daqp.cycles_[0]);
	printf("Cycles[1]: %u\n",  daqp.cycles_[1]);
	printf("Cycles[2]: %u\n",  daqp.cycles_[2]);
	printf("Cycles[3]: %u\n",  daqp.cycles_[3]);

	printf("tn[0]: %u\n",  daqp.tn_[0]);
	printf("tn[1]: %u\n",  daqp.tn_[1]);
	printf("tn[2]: %u\n",  daqp.tn_[2]);
	printf("tn[3]: %u\n",  daqp.tn_[3]);

	printf("Seqnum: %u\n",  daqp.seqnum_);
	printf("Pkt_Length: %u\n",  daqp.pkt_length_);
#endif

#ifdef DAQTIMERUW_EE
	printf("Time: %u\n",  daqp.time_);
	printf("Packet length: %u\n",  daqp.length_);
#endif

#ifdef DAQTIMERUW
	printf("Time: %u\n",  daqp.time_);
	printf("Packet length: %u\n",  daqp.length_);
#endif

#ifdef DAQTRACKING
	cerr << "Centroid = (" << daqp.position_[0] << ", " << daqp.position_[1] <<  ")" << endl;
#endif

#ifdef DAQMIRTES
	printf("Counter: %u\n",  daqp.counter_);
	printf("Minimum: %u\n",  daqp.minimum_);
	printf("Maximum: %u\n",  daqp.maximum_);
	printf("Average: %f\n",  daqp.average_);
#endif

#ifdef DAQSTEREO
	printf("DAQSData::printData():");

	cout << "Image_sx: " << endl;
	for (int k =0; k < STEREO_BUFFER; k++){
		printf("0x%x ", daqp.image_sx_[k]);
		if (k % 10 == 0) printf("\n");
	}
	getchar();

	cout << "Image_dx: " << endl;
	for (int k =0; k < STEREO_BUFFER; k++){
		printf("0x%x ", daqp.image_dx_[k]);
		if (k % 10 == 0) printf("\n");
	}
	getchar();

	cout << "Image_stereo: " << endl;
	for (int k =0; k < STEREO_BUFFER; k++){
		printf("0x%x ", daqp.image_stereo_[k]);
		if (k % 10 == 0) printf("\n");
	}
	getchar();
#endif
}
//______________________________________________________________________________
void DAQSData::Clear(Option_t * /*option*/)
{
}

#ifdef DAQSTEREO
//Create the histogram for the actual image
void create_histogram(unsigned char *image, unsigned long size, unsigned char *h){
	unsigned long i = 0;

	for (i = 0; i < size; i++)
		*(h + image[i]) += 1;
}
#endif
