/*
Copyright (C) 2009 -  Claudio Salvadori 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

 /**
 *\file CImage.cpp
 *\brief  Image rendering class method development.
 */

#include "CImage.h"
#include <myUtils.h>
#include <iostream>

#define EXTRA	30

IRenderer::Factory registerFactoryI("image", CImage::createObj);

class ImageException : public std::exception {
public:
        ImageException(const char *msg) throw() : errMessage(msg) { } ;
        ~ImageException(void) throw() { } ;
        const char *what(void) const throw() { return errMessage.c_str(); };
private:
        std::string errMessage;
       	ImageException(void) throw();
};

IRenderer* CImage::createObj(const void* node){
	return new CImage((const DOMNode*)node);
}

CImage::CImage(const DOMNode* node){
	parser_ = new CXMLParser;

	imagePreset((const DOMNode*) node);
	
	screen_wnd = (Fl_Double_Window *) 0;
	screen_img = (Fl_Box *) 0;
	
	Fl::visual(FL_RGB);
	{
		screen_wnd = new Fl_Double_Window(w_ + EXTRA , h_ + EXTRA, name_.c_str());
		screen_wnd->box(FL_BORDER_BOX);
		{
			screen_img = new Fl_Box(EXTRA / 2, EXTRA / 2, w_, h_);
			screen_img->box(FL_FLAT_BOX);
		}
		screen_wnd->end();
	}
	
	screen_wnd->show();
	Fl::check();	
} 

CImage::~CImage(){
	if(screen_wnd) delete screen_wnd;
	if(screen_img) delete screen_img;
	
	if(inode_) delete inode_;
	if(parser_) delete parser_;	
}

void CImage::imagePreset(const DOMNode* node){
	char ret[256];
	
	if(!parser_->FindNodeRootat((DOMNode*)node, (char*)"image_preset", inode_)){
		throw ImageException("image_preset node isn't found");
	}else{
		if (!parser_->GetAttribute(inode_, (char*)"image_name", (char*)ret)){
			throw ImageException("image_name attribute isn't found");
		}else{
			#ifdef DEBUG
			printf("Image name returned: %s\n", ret);
			#endif
			name_ = ret;		/* Image renderer name*/
			IRenderer::checkName(name_);
		}
		
		if (!parser_->GetAttribute(inode_, (char*)"image_width", (char*)ret)){
			throw ImageException("image_width attribute isn't found");
		}else{
			#ifdef DEBUG
			printf("Image width returned: %s\n", ret);
			#endif
			w_ = atol(ret);		/* Image width (in pixel)*/
		}
		
		if (!parser_->GetAttribute(inode_, (char*)"image_height", (char*)ret)){
			throw ImageException("image_height attribute isn't found");
		}else{
			#ifdef DEBUG
			printf("Image height returned: %s\n", ret);
			#endif
			h_ = atol(ret);		/* Image height (in pixel)*/
		}
		
		if (!parser_->GetAttribute(inode_, (char*)"image_channel", (char*)ret)){
			throw ImageException("image_channel attribute isn't found");
		}else{
			#ifdef DEBUG
			printf("Image color channels returned: %s\n", ret);
			#endif
			channel_ = atoi(ret);		/* Number of color channels*/
		}
	}
}

void CImage::render(void* data, unsigned long size){
	fflush(stdout);
	static Fl_RGB_Image*	m_image;
	
	if(size == w_ * h_){
		m_image = new Fl_RGB_Image((unsigned char*)data, w_, h_, channel_);
		screen_img->image(m_image);
		screen_img->redraw();
		Fl::check();
	
		delete m_image;
	}else
		throw ImageException("Uncorrect image size");
}

