/*
Copyright (C) 2009 -  Claudio Salvadori
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/**
 *\file CPipe.cpp
 *\brief  Pipe access functions develpoment
 */
#include "CPipe.h"

#define MAX_ACK_LENGHT 100

using namespace std;

CPipe::~CPipe(){
	Pipe_close();
}

s_ret CPipe::Pipe_handshaking(handshake_style style){
	s_ret help;
	unsigned char data[MAX_ACK_LENGHT];
	
	//handshaking if it exists...
	switch(style){
		case H_S_STANDARD:
		help = Pipe_write((unsigned char*)(*pi_Handshk)[0].h_Pkt, (*pi_Handshk)[0].h_Size_P);
		if(help != S_RET_OK)
			return help;
		break;
		
		case H_S_NONE:
		return S_RET_OK;
		break;
		
		case H_S_START:
		for(unsigned int i = 0; i < pi_Handshk->size(); i++){
			//send handshking message
			help = Pipe_write((unsigned char*)(*pi_Handshk)[i].h_Pkt, (*pi_Handshk)[i].h_Size_P);
			if(help != S_RET_OK)
				return help;

			if(((*pi_Handshk)[i].h_Size_A != 0) && ((*pi_Handshk)[i].h_Size_N != 0)){
							
				//reading answer
				help = Pipe_read(data, (*pi_Handshk)[i].h_Size_A);
				if(help != S_RET_OK)
					return help;

				if(memcmp((*pi_Handshk)[i].h_Ack, data, (*pi_Handshk)[i].h_Size_A) != 0){
					if(memcmp((*pi_Handshk)[i].h_Nack, data, (*pi_Handshk)[i].h_Size_N) == 0)
						return S_P_NACK;
					else 
						return S_P_GEN_ERR;
				}
			}
		}
		break;
	}
	
	return S_RET_OK;
}
	
s_ret CPipe::getDefinition (struct SDef def, s_read_style sty){
	
	pi_Name = (char *) def.s_Name.c_str();		
	pi_Style = sty;
	
	switch(pi_Style){
		case S_STYLE_LENGTH:
		pi_Size = def.s_Size;
		#ifdef DEBUG
		printf("Lenght style\n");
		#endif
		break;
	
		case S_STYLE_S_E:
		pi_Start = (unsigned char)def.s_Option->o_Start;
		pi_End = (unsigned char)def.s_Option->o_End;
		#ifdef DEBUG
		printf("Start/end style \n");
		#endif
		break;
	}
	
	pi_HStyle = def.s_Style;
	
	if(pi_HStyle != H_S_NONE)
		pi_Handshk = def.s_Handshk;

	if(pi_HStyle == H_S_START){
		s_ret help;
		help = Pipe_init();
		if(help != S_RET_OK)
			return help;
		
		help = Pipe_handshaking(pi_HStyle);
		
		//Handshake did! disable all the eventual future possibilities to have an hanshake!
		pi_HStyle = H_S_NONE;
		
		return help;
	}else{
		return Pipe_init();
	}

	return S_RET_OK;
}

//Blocking function
s_ret CPipe::retStream(unsigned char **buffer, unsigned long *length){
	s_ret help;
	switch(pi_Style){		
		case S_STYLE_LENGTH:
		if(pi_HStyle == H_S_STANDARD){
			help = Pipe_handshaking(pi_HStyle);
			if(help != S_RET_OK)
				return help;
		}
		
		*buffer = new unsigned char[pi_Size];
		help = Pipe_read(*buffer, (unsigned int) pi_Size);
		*length = pi_Size_r;
		
		#ifdef DEBUG
		for(unsigned long i = 0; i < *length; i++)
			printf("Character received: %x, %d\n", (int)(*buffer)[i], (int)(*buffer)[i]);
		#endif
		
		return help;
		break;
		
		case S_STYLE_S_E:
		std::vector<unsigned char> buff;
		unsigned char buff_t;
		unsigned int size = 1;
		
		if(pi_HStyle == H_S_STANDARD){
			help = Pipe_handshaking(pi_HStyle);
			if(help != S_RET_OK)
				return help;
		}
		
		//Searching for syncro character (start character)...
		while(buff_t != pi_Start){				
			help = Pipe_read(&buff_t, size);
			if(help != S_RET_OK)
				return help;
			#ifdef DEBUG
			printf("%d", buff_t);
			#endif
		}
		
		//clear the buffer
		buff.clear();

		//... and reading untill end character
		while(1){
			help = Pipe_read(&buff_t, size);
			if(help != S_RET_OK)
				return help;
			
			#ifdef DEBUG
			printf("%d", buff_t);
			#endif
			
			//End character found	
			if(buff_t == pi_End){
				*length = buff.size();
				*buffer = new unsigned char[*length];
				
				for(unsigned long i_s = 0; i_s < *length; i_s++) 
					(*buffer)[i_s] = buff[i_s];
				
				#ifdef DEBUG
				for(unsigned long i_s = 0; i_s < *length; i_s++){
					printf("Character received: %x, %d\n", (int)(*buffer)[i_s], (int)(*buffer)[i_s]);
					printf("Character received: %x, %d\n", (int)buff[i_s], (int)buff[i_s]);
				}
				#endif
				return S_RET_OK;
			}
	
			//push back the character
			buff.push_back(buff_t);
		};
		break;
	}
	
	return S_RET_OK;
}

s_ret CPipe::Pipe_init(void){
	#if defined(__WIN32__) || defined(__CYGWIN__) /* WIN32 Implementation */
	BOOL fSuccess;
	while (1){ 
		pi_Handler = CreateFile(
			pi_Name,   		// pipe name 
			GENERIC_READ |  // read and write access 
			GENERIC_WRITE, 
			0,              // no sharing 
			NULL,           // default security attributes
			OPEN_EXISTING,  // opens existing pipe 
			0,              // default attributes 
			NULL);          // no template file 

		// Break if the pipe handle is valid. 
		if (pi_Handler != INVALID_HANDLE_VALUE) 
			break; 

		// Exit if an error other than ERROR_PIPE_BUSY occurs. 
		if (GetLastError() != ERROR_PIPE_BUSY)
			return S_P_INIT_ERR;
 
		// All pipe instances are busy, so wait for 20 seconds. 
		if (!WaitNamedPipe(pi_Name, 20000)) 
	        	return S_P_INIT_ERR;
	} 
	
	fSuccess = SetNamedPipeHandleState( 
		pi_Handler, 	// pipe handle 
		NULL,  		// new pipe mode 
		NULL,     	// don't set maximum bytes 
		NULL);    	// don't set maximum time 
	if (!fSuccess) 
		return S_P_INIT_ERR;
	else
		return S_RET_OK;
	#else 									/* Unix-like Implementation */
		return S_P_NOTIMPL;
	#endif
	return S_RET_OK;
}

void CPipe::Pipe_close(void){
#if defined(__WIN32__) || defined(__CYGWIN__) 	/* WIN32 Implementation */
	CloseHandle(pi_Handler);
#else											/* Unix-like Implementation */
	return;
#endif
}

s_ret CPipe::Pipe_read(unsigned char *data, unsigned int data_size){
#if defined(__WIN32__) || defined(__CYGWIN__) 	/* WIN32 Implementation */
	DWORD read_size;
	if( ReadFile(pi_Handler, data, sizeof(char)*data_size, &read_size, NULL) == FALSE )
		return S_P_READ_ERR;
	else{	
		pi_Size_r = (unsigned int)read_size;
		if((unsigned int)read_size == data_size){
			return S_RET_OK;
		}else{
			if(read_size < 0)
				return S_P_P_CHAR_ERR;
			else if(read_size == 0)
				return S_P_R_TIMEOUT_ERR;
			else return S_P_LENGHT_ERR;
		}
	}	
#else 											/* Unix-like Implementation */
	return S_P_NOTIMPL; 
#endif

	return S_RET_OK;
}

s_ret CPipe::Pipe_write(unsigned char* data, unsigned int data_size){
#if defined(__WIN32__) || defined(__CYGWIN__) /* WIN32 Implementation */
	DWORD write_size;
	if( !WriteFile(pi_Handler, data, sizeof(char)*data_size, &write_size, NULL) )	
		return S_P_WRITE_ERR;
	else{
		if((unsigned int)write_size == data_size)
			return S_RET_OK;
		else
			if(write_size < 0)
				return S_P_G_CHAR_ERR;
			else if(write_size == 0)
				return S_P_T_TIMEOUT_ERR;
			else return S_P_LENGHT_ERR;
	}
#else 										/* Unix-like Implementation */
	return S_P_NOTIMPL;
#endif
	return S_RET_OK;
}

